@@ -24,10 +24,10 @@ from .commands.decorators import (
     require_store,
     save_store_changes,
 )
-from .commands.filtering import get_filter_from_command_arguments
+from .commands.filtering import Filter
 from .commands.helpers import abort_if_false, display_pod_store_error_from_exception
-from .commands.listing import get_lister_from_command_arguments
-from .commands.tagging import get_tagger_from_command_arguments
+from .commands.listing import Lister
+from .commands.tagging import Tagger
 from .store import Store
 from .store_file_handlers import EncryptedStoreFileHandler, UnencryptedStoreFileHandler
 from .util import run_git_command
@@ -208,26 +208,19 @@ def download(
     tagged = list(tagged or [])
     untagged = list(untagged or [])
 
-    if episode is not None:
-        new_episodes = False
-        filters = {"episode_number": episode}
-    else:
-        new_episodes = True
-        filters = {}
-
-    filter = get_filter_from_command_arguments(
+    pod_store_filter = Filter.from_command_arguments(
         store=store,
-        new_episodes=new_episodes,
+        new_episodes=True,
         filter_for_episodes=True,
         podcast_title=podcast,
+        episode_number=episode,
         tagged=tagged,
         untagged=untagged,
         podcasts_tagged=podcast_tagged,
         podcasts_untagged=podcast_untagged,
-        **filters,
     )
 
-    for ep in filter.items:
+    for ep in pod_store_filter.items:
         click.echo(f"Downloading: {ep.download_path}.")
         try:
             with ep.download() as download:
@@ -349,7 +342,7 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
 )
 @click.option(
     "--episodes/--podcasts",
-    default=False,
+    default=None,
     help="(flag): List episodes or podcasts. Defaults to `--podcasts`.",
 )
 @click.option(
@@ -390,7 +383,7 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
 def ls(
     ctx: click.Context,
     new: bool,
-    episodes: bool,
+    episodes: Optional[bool],
     podcast: Optional[str],
     episode: Optional[int],
     tagged: Optional[List[str]],
@@ -402,29 +395,25 @@ def ls(
     By default, this will list podcasts that have new episodes. Adjust the output using
     the provided flags and command options.
     """
+    # It is important to set the `--episodes/--podcasts` option to `None` by default,
+    # rather than `False`. A null value indicates that it has not been set by
+    # explicitly the user, which triggers behavior for inferring the value from other
+    # command arguments.
+
     store = ctx.obj
     tagged = list(tagged or [])
     untagged = list(untagged or [])
 
-    if episode is not None:
-        list_episodes = True
-        new_episodes = False
-        filters = {"episode_number": episode}
-    else:
-        list_episodes = bool(episodes or podcast)
-        new_episodes = new
-        filters = {}
-
-    lister = get_lister_from_command_arguments(
+    lister = Lister.from_command_arguments(
         store=store,
-        new_episodes=new_episodes,
-        list_episodes=list_episodes,
+        new_episodes=new,
+        list_episodes=episodes,
         podcast_title=podcast,
+        episode_number=episode,
         tagged=tagged,
         untagged=untagged,
-        **filters,
     )
-    for msg in lister.list(verbose=verbose):
+    for msg in lister.list_items(verbose=verbose):
         click.echo(msg)
 
 
@@ -442,6 +431,29 @@ def ls(
     help="(flag): Run this command in interactive mode to select which episodes to "
     "mark, or bulk mode to mark all episodes. Defaults to `--interactive`.",
 )
+@click.option(
+    "-s",
+    "--start",
+    type=int,
+    default=None,
+    help=(
+        "Starting episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option(
+    "-e",
+    "--end",
+    type=int,
+    default=None,
+    help=(
+        "Ending episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option("-r", "--reverse", is_flag=True, help="Reverse order of items.")
 @click.option(
     "-f",
     "--force",
@@ -462,19 +474,26 @@ def mark_as_new(
     ctx: click.Context,
     podcast: Optional[str],
     interactive: bool,
+    start: Optional[int],
+    end: Optional[int],
+    reverse: bool,
     force: Optional[bool],
 ):
     """Add the `new` tag to a group of episodes."""
     store = ctx.obj
-    tagger = get_tagger_from_command_arguments(
+    episode_range_start = start
+    episode_range_end = end
+    tagger = Tagger.from_command_arguments(
         store=store,
         tags=["new"],
         tag_episodes=True,
+        episode_range_start=episode_range_start,
+        episode_range_end=episode_range_end,
         podcast_title=podcast,
         action="mark",
     )
 
-    for msg in tagger.tag_items(interactive_mode=interactive):
+    for msg in tagger.tag_items(interactive_mode=interactive, reverse_order=reverse):
         click.echo(msg)
 
 
@@ -492,6 +511,29 @@ def mark_as_new(
     help="(flag): Run this command in interactive mode to select which episodes to "
     "mark, or bulk mode to mark all episodes. Defaults to `--interactive`.",
 )
+@click.option(
+    "-s",
+    "--start",
+    type=int,
+    default=None,
+    help=(
+        "Starting episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option(
+    "-e",
+    "--end",
+    type=int,
+    default=None,
+    help=(
+        "Ending episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option("-r", "--reverse", is_flag=True, help="Reverse order of items.")
 @click.option(
     "-f",
     "--force",
@@ -512,20 +554,28 @@ def mark_as_old(
     ctx: click.Context,
     podcast: Optional[str],
     interactive: bool,
+    start: Optional[int],
+    end: Optional[int],
+    reverse: bool,
     force: Optional[bool],
 ):
     """Remove the `new` tag from a group of episodes."""
+    episode_range_start = start
+    episode_range_end = end
     store = ctx.obj
-    tagger = get_tagger_from_command_arguments(
+
+    tagger = Tagger.from_command_arguments(
         store=store,
         tags=["new"],
         tag_episodes=True,
+        episode_range_start=episode_range_start,
+        episode_range_end=episode_range_end,
         podcast_title=podcast,
         is_untagger=True,
         action="unmark",
     )
 
-    for msg in tagger.tag_items(interactive_mode=interactive):
+    for msg in tagger.tag_items(interactive_mode=interactive, reverse_order=reverse):
         click.echo(msg)
 
 
@@ -591,15 +641,16 @@ def refresh(
     if not podcast:
         untagged.append("inactive")
 
-    filter = get_filter_from_command_arguments(
+    pod_store_filter = Filter.from_command_arguments(
         store=store,
+        new_episodes=False,
         filter_for_episodes=False,
         podcast_title=podcast,
         tagged=tagged,
         untagged=untagged,
     )
-    for podcast in filter.items:
-        click.echo(f"Refreshing {podcast.title}")
+    for podcast in pod_store_filter.items:
+        click.echo(f"Refreshing {podcast.title}.")
         try:
             podcast.refresh()
         except requests.ConnectTimeout:
@@ -689,7 +740,6 @@ def set_inactive(ctx: click.Context, podcast: str):
     "--podcast", "-p", default=None, help="(podcast title): Tag a single podcast."
 )
 @click.option(
-    "-e",
     "--episode",
     type=int,
     default=None,
@@ -705,6 +755,29 @@ def set_inactive(ctx: click.Context, podcast: str):
     default=True,
     help="Interactively determine which items to tag, or apply the tag to all items.",
 )
+@click.option(
+    "-s",
+    "--start",
+    type=int,
+    default=None,
+    help=(
+        "Starting episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option(
+    "-e",
+    "--end",
+    type=int,
+    default=None,
+    help=(
+        "Ending episode number for episode range. "
+        "Note that you must specify a podcast these episodes belong to using the "
+        "`--podcast` option."
+    ),
+)
+@click.option("-r", "--reverse", is_flag=True, help="Reverse order of items.")
 @click.option(
     "-f",
     "--force",
@@ -729,6 +802,9 @@ def tag(
     episode: Optional[int],
     episodes: bool,
     interactive: bool,
+    start: Optional[int],
+    end: Optional[int],
+    reverse: bool,
     force: bool,
 ):
     """Tag (or untag) podcasts or episodes with arbitrary tags.
@@ -748,22 +824,20 @@ def tag(
     options. Specify a podcast by title to tag the podcast. To tag an episode, specify
     the podcast by title and then specify the episode by episode number.
     """
-    tag_episodes = bool(not podcast or episode) and bool(episodes or episode)
-    store = ctx.obj
-    if episode is not None:
-        filters = {"episode_number": episode}
-    else:
-        filters = {}
+    tag_episodes = bool(podcast and episode or start or end) or (
+        not podcast and episodes
+    )
 
-    tagger = get_tagger_from_command_arguments(
+    store = ctx.obj
+    tagger = Tagger.from_command_arguments(
         store=store,
         tags=tag,
         tag_episodes=tag_episodes,
         podcast_title=podcast,
+        episode_number=episode,
         is_untagger=untag,
-        filters=filters,
     )
-    for msg in tagger.tag_items(interactive_mode=interactive):
+    for msg in tagger.tag_items(interactive_mode=interactive, reverse_order=reverse):
         click.echo(msg)
 
 
