@@ -8,7 +8,7 @@ Reading/writing to the store file is delegated to the classes in the
 import os
 from typing import List, Optional
 
-from . import GPG_ID_FILE_PATH, PODCAST_DOWNLOADS_PATH
+from . import GPG_ID_FILE_PATH, PODCAST_DOWNLOADS_PATH, get_store_file_path
 from .exc import (
     PodcastDoesNotExistError,
     PodcastExistsError,
@@ -130,7 +130,7 @@ class Store:
     def init(
         cls,
         store_path: str,
-        store_file_path: str,
+        store_file_name: str,
         setup_git: bool,
         git_url: Optional[str] = None,
         gpg_id: Optional[str] = None,
@@ -142,35 +142,46 @@ class Store:
         Optionally set the GPG ID for store encryption and establish the store file
         as an encrypted file.
         """
-        if git_url:
-            return cls._setup_existing_repo(git_url, store_path, gpg_id=gpg_id)
+        store_file_path = os.path.join(store_path, store_file_name)
+        gitignore_file_path = os.path.join(store_path, ".gitignore")
 
-        try:
-            os.makedirs(store_path)
-        except FileExistsError:
-            raise StoreExistsError(store_path)
         os.makedirs(PODCAST_DOWNLOADS_PATH, exist_ok=True)
 
-        if setup_git:
-            run_git_command("init")
-            with open(os.path.join(store_path, ".gitignore"), "w") as f:
-                f.write(".gpg-id")
+        if git_url:
+            cls._setup_existing_repo(git_url, store_path, gpg_id=gpg_id)
+        else:
+            try:
+                os.makedirs(store_path)
+            except FileExistsError:
+                raise StoreExistsError(store_path)
+
+            if setup_git:
+                run_git_command("init")
+
+                with open(gitignore_file_path, "w") as f:
+                    f.write(".gpg-id")
 
         if gpg_id:
             cls._setup_encrypted_store(gpg_id=gpg_id, store_file_path=store_file_path)
         else:
-            UnencryptedStoreFileHandler.create_store_file(store_file_path)
+            cls._setup_unencrypted_store(store_file_path)
 
     def __repr__(self) -> str:
         return f"<Store({self._store_path!r})>"
 
     def encrypt(self, gpg_id: str) -> None:
         """Encrypt an existing store that is currently stored in plaintext."""
-        store_file_path = self._file_handler.store_file_path
+        existing_store_file_path = self._file_handler.store_file_path
         store_data = self._file_handler.read_data()
+        encrypted_store_file_path = get_store_file_path(gpg_id=gpg_id)
         self._setup_encrypted_store(
-            gpg_id=gpg_id, store_file_path=store_file_path, store_data=store_data
+            gpg_id=gpg_id,
+            store_file_path=encrypted_store_file_path,
+            store_data=store_data,
+            overwrite_existing=True,
         )
+        if encrypted_store_file_path != existing_store_file_path:
+            os.remove(existing_store_file_path)
 
     def unencrypt(self) -> None:
         """Unencrypt an existing store that is currently stored as encrypted data.
@@ -180,12 +191,16 @@ class Store:
         """
         if not os.path.exists(GPG_ID_FILE_PATH):
             raise StoreIsNotEncrypted(GPG_ID_FILE_PATH)
-        store_file_path = self._file_handler.store_file_path
+        existing_store_file_path = self._file_handler.store_file_path
         store_data = self._file_handler.read_data()
-        UnencryptedStoreFileHandler.create_store_file(
-            store_file_path=store_file_path, store_data=store_data
+        unencrypted_store_file_path = get_store_file_path(gpg_id=None)
+        self._setup_unencrypted_store(
+            store_file_path=unencrypted_store_file_path,
+            store_data=store_data,
+            overwrite_existing=True,
         )
-        os.remove(GPG_ID_FILE_PATH)
+        if existing_store_file_path != unencrypted_store_file_path:
+            os.remove(existing_store_file_path)
 
     def save(self) -> None:
         """Save data to the store json file."""
@@ -203,7 +218,10 @@ class Store:
 
     @staticmethod
     def _setup_encrypted_store(
-        gpg_id: str, store_file_path: str, store_data: dict = None
+        gpg_id: str,
+        store_file_path: str,
+        store_data: dict = None,
+        overwrite_existing: bool = False,
     ) -> None:
         """Set up the store as a GPG encrypted store.
 
@@ -214,6 +232,22 @@ class Store:
 
         with open(os.path.join(GPG_ID_FILE_PATH), "w") as f:
             f.write(gpg_id)
-        EncryptedStoreFileHandler.create_store_file(
-            gpg_id=gpg_id, store_file_path=store_file_path, store_data=store_data
-        )
+
+        if not os.path.exists(store_file_path) or overwrite_existing:
+            EncryptedStoreFileHandler.create_store_file(
+                gpg_id=gpg_id, store_file_path=store_file_path, store_data=store_data
+            )
+
+    @staticmethod
+    def _setup_unencrypted_store(
+        store_file_path: str, store_data: dict = None, overwrite_existing: bool = False
+    ) -> None:
+        store_data = store_data or {}
+
+        if not os.path.exists(store_file_path) or overwrite_existing:
+            UnencryptedStoreFileHandler.create_store_file(
+                store_file_path, store_data=store_data
+            )
+
+        if os.path.exists(GPG_ID_FILE_PATH):
+            os.remove(GPG_ID_FILE_PATH)
